@@ -62,9 +62,11 @@ class cognet:
         """
         if model is not None:
             self.qnet = model.myQnet
-            self.cols = np.array(model.features)
+            # self.cols = np.array(model.features)
             featurenames, samples = data_obj.format_samples(key)
-            self.features = pd.DataFrame(columns=self.cols)
+            samples = pd.DataFrame(samples)
+            self.cols = np.array(featurenames)
+            self.features = pd.DataFrame(columns=np.array(featurenames))
             if any(x is not None for x in [model.immutable_vars, model.mutable_vars]):
                 if model.immutable_vars is not None:
                     self.immutable_vars = model.immutable_vars
@@ -75,9 +77,10 @@ class cognet:
             else:
                 self.mutable_vars = self.features
             
-            self.samples = pd.concat([samples,self.features], axis=0)
+            self.samples = pd.DataFrame(samples)
+            self.samples.columns = np.array(featurenames)
             self.all_samples = self.samples
-            self.samples_as_strings = self.samples[self.cols].fillna('').values.astype(str)[:]
+            self.samples_as_strings = self.samples.fillna('').values.astype(str)[:]
             self.s_null=['']*len(self.samples_as_strings[0])
             self.D_null=self.qnet.predict_distributions(self.s_null)
             variation_weight = []
@@ -96,11 +99,14 @@ class cognet:
         Args:
           data_obj (class): instance of dataformatter class
           key (str): 'all', 'train', or 'test', corresponding to sample type
+          
+        Returns:
+          featurenames, samples: formatted arrays
         """
         featurenames, samples = data_obj.format_samples(key)
         if any(x is not None for x in [self.features, self.samples]):
             print("replacing original features/samples with dataformatter data")
-        self.cols = np.array(model.features)
+        self.cols = featurenames
         self.features = pd.DataFrame(columns=self.cols)
         self.samples = pd.DataFrame(samples,columns=self.features)
         self.all_samples = self.samples
@@ -162,30 +168,37 @@ class cognet:
                             if x.upper() not in self.immutable_vars.columns]
     
     def set_nsamples(self,
-                    num_samples):
+                    num_samples=None,
+                    random=True):
         '''select a subset of the samples
 
         Args:
-          num_samples (int): Set num of samples to subset
+          num_samples (int): Set num of samples to subset, default to None, resets to all samples
+          random (bool): take random sample if true, ordered sample if false
         '''
         self.samples = self.all_samples
-        if all(x is not None for x in [num_samples, self.samples]):
-            if num_samples > len(self.samples.index):
-                string = 'The number of selected samples ({}) ' + \
-                    'is greater than the number of samples ({})!'
-                string = string.format(num_samples, len(self.samples.index))
-                raise ValueError(string)
+        if num_samples is not None:
+            if all(x is not None for x in [num_samples, self.samples]):
+                if num_samples > len(self.samples.index):
+                    string = 'The number of selected samples ({}) ' + \
+                        'is greater than the number of samples ({})!'
+                    string = string.format(num_samples, len(self.samples.index))
+                    raise ValueError(string)
 
-            if num_samples == len(self.samples.index):
-                string = 'The number of selected samples ({}) ' + \
-                    'is equal to the number of samples ({})!'
-                string = string.format(num_samples, len(self.samples.index))
-                print(string)
-            self.samples = self.samples.sample(num_samples)
-            self.samples_as_strings = self.samples[self.cols].fillna('').values.astype(str)[:]
-
-        elif self.samples is None:
-            raise ValueError("load_data first!")
+                if num_samples == len(self.samples.index):
+                    string = 'The number of selected samples ({}) ' + \
+                        'is equal to the number of samples ({})!'
+                    string = string.format(num_samples, len(self.samples.index))
+                    print(string)
+                if random:
+                    self.samples = self.samples.sample(num_samples)
+                   
+                else:
+                    self.samples = self.samples.iloc[:num_samples]
+                self.samples_as_strings = self.samples[self.cols].fillna('').values.astype(str)[:]
+                
+            elif self.samples is None:
+                raise ValueError("load_data first!")
 
     def __variation_weight(self,
                         index):
@@ -302,7 +315,7 @@ class cognet:
           nsteps2 (int, optional): number of steps to qsample for sample2
 
         Returns:
-          float: qdistance
+          qdistance: float, distance between two samples
         """
         if self.qnet is None:
             raise ValueError("load qnet first!")
@@ -320,6 +333,9 @@ class cognet:
         Args:
           x (list[str]): first sample
           y (list[str]): second sample
+          
+        Returns:
+         d: qdistance
         '''
         d=qdistance(x,y,self.qnet,self.qnet)
         return d
@@ -330,11 +346,11 @@ class cognet:
         """return the distance from a sample to the poles
 
         Args:
-            i (int): index of sample to take
-            return_dict (dict): dict used for multiple sample function
+          i (int): index of sample to take
+          return_dict (dict): dict used for multiple sample function
 
         Returns:
-            [type]: [description]
+          distances: float, distance from sample to each pole
         """
         samples_as_strings = self.samples[self.cols].fillna('').values.astype(str)[:]
         p = samples_as_strings[i]
@@ -352,6 +368,9 @@ class cognet:
 
         Args:
           outfile (str): desired output filename and path
+          
+        Returns:
+          return_dict: dictionary containing multiprocessing results
         """
         if all(x is not None for x in [self.samples, self.cols,
                                     self.polar_features]):
@@ -385,7 +404,7 @@ class cognet:
           i (int): row
         
         Return:
-          numpy.ndarray(float)
+          line: float, numpy.ndarray
         '''
         if all(x is not None for x in [self.samples, self.features]):
             w = self.samples.index.size
@@ -408,6 +427,9 @@ class cognet:
 
         Args:
           outfile (str): desired output filename and path
+          
+        Returns:
+          return_dict: dictionary containing multiprocessing results
         """
         if all(x is not None for x in [self.samples, self.features]):
             manager = mp.Manager()
@@ -428,14 +450,18 @@ class cognet:
             result.to_csv(outfile)
         else:
             raise ValueError("load data first!")
-        return return_dict
+        
+        return pd.DataFrame(return_dict.copy())
     
     def polar_separation(self,
                         nsteps=0):
-        """returns the distance between poles as a qdistance matrix
+        """calculates the distance between poles as a qdistance matrix
 
         Args:
           nsteps (int, optional): [description]. Defaults to 0.
+          
+        Returns:
+          self.polar_matrix: dictionary containing multiprocessing results
         """
         polar_arraydata = self.polar_features[self.cols].fillna('').values.astype(str)[:]
         samples_ = []
@@ -490,8 +516,8 @@ class cognet:
         """calculate distance between two poles
 
         Args:
-            pole_1 (list[str]): a polar vector, must have same number of features as qnet
-            pole_2 (list[str]): a polar vector, must have same number of features as qnet
+          pole_1 (list[str]): a polar vector, must have same number of features as qnet
+          pole_2 (list[str]): a polar vector, must have same number of features as qnet
         """
         self.pL = self.poles_dict[pole_1]
         self.pR = self.poles_dict[pole_2]
@@ -509,6 +535,12 @@ class cognet:
           pole_1 (int): index of Pole One to calc as base distance. Defaults to 0.
           pole_2 (int): index of Pole Two to calc as base distance. Defaults to 1.
           return_dict (dict, optional): dict containing results
+          
+        Returns:
+          [ideology_index, dR, dL, self.d0]: which way the sample leans,
+                                             distance from the right pole,
+                                             distance from the left pole,
+                                             and distance between poles, respectively
         """
         if pole_1 is not None or pole_2 is not None:
             self.__calc_d0(pole_1, pole_2)
@@ -528,11 +560,11 @@ class cognet:
         to determine max and std of distances between qsamples
 
         Args:
-            i (int): index of sample
-            return_dict (dict): dict containing results
+          i (int): index of sample
+          return_dict (dict): dict containing results
 
         Returns:
-            list[float]: std and max of the distances btwn qsamples
+          list[float]: std and max of the distances btwn qsamples
         """
         p = self.samples_as_strings[i]
         Qset = [qsample(p, self.qnet, self.steps) for j in np.arange(self.num_qsamples)]
@@ -565,8 +597,11 @@ class cognet:
           pole_2 (int, optional): index of Pole Two to calc as base distance. Defaults to 1.
 
         Raises:
-            ValueError: set poles if poles are not set
-            ValueError: load data if samples or features are not present
+          ValueError: set poles if poles are not set
+          ValueError: load data if samples or features are not present
+            
+        Returns:
+          Result: dictionary containing multiprocessing results
         """
         if all(x is not None for x in [self.samples, self.features,
                                     self.pL, self.pR]):
@@ -604,7 +639,7 @@ class cognet:
             raise ValueError("set_poles first!")
         else:
             raise ValueError("load_data first!")
-        return result
+        return pd.DataFrame(return_dict.copy())
 
     def compute_polar_indices(self,
                             num_samples = None,
@@ -645,6 +680,9 @@ class cognet:
           sample_index (int): index of the sample to compute dissonance
           return_dict (dict): dict containing results
           MISSING_VAL (float): default dissonance value
+          
+        Returns: 
+          diss: ndarray containing dissonance for sample
         '''
         if all(x is not None for x in [self.samples, self.features]):
             s = self.samples_as_strings[sample_index]
@@ -707,6 +745,10 @@ class cognet:
 
         Args:
           X (1D array-like): vector from which random element is to be chosen
+        
+        Returns:
+          X: random element of sample
+          None: if X has len 0
         '''
         X=list(X)
         if len(X)>0:
@@ -723,10 +765,19 @@ class cognet:
           s (list[str]): vector of sample, must have the same num of features as the qnet
           mask_prob (float): float btwn 0 and 1, prob to mask element of sample
           allow_all_mutable (bool): whether or not all variables are mutable
+          
+        Returns:
+          s1,base_frequency,MASKrand,
+          np.where(base_frequency)[0],
+          np.mean(rnd_match_prob),
+          np.mean(max_match_prob),
+          s_rand
         '''
-        if self.samples is not None:   
+        if self.samples is not None:
+            s0=s.copy()
+            s0=np.array(s0)   
             MUTABLE=pd.DataFrame(np.zeros(len(self.cols)),index=self.cols).transpose()
-            WITHVAL=[x for x in self.cols[np.where(s)[0]] if x in self.mutable_vars ]
+            WITHVAL=[x for x in self.cols[np.where(s0)[0]] if x in self.mutable_vars ]
             MASKrand=[x for x in WITHVAL if random.random() < mask_prob ]
             for m in MASKrand:
                 MUTABLE[m]=1.0
@@ -784,9 +835,6 @@ class cognet:
         Raises:
           ValueError: Neither sample or index were given
           ValueError: Both sample and index were given
-
-        Returns:
-          [type]: [description]
         """
         if all(x is None for x in [sample, index]):
             raise ValueError("Must input either sample or index!")
@@ -821,6 +869,9 @@ class cognet:
 
         Args:
           output_file (str): directory and/or file for output
+          
+        Returmns:
+          result.rederr.mean(), result.rand_err.mean(): mean of reconstruction error and random error
         '''
         manager = mp.Manager()
         return_dict = manager.dict()
@@ -851,7 +902,7 @@ class cognet:
         else:
             result.to_csv(out_file)
         
-        return result.rederr.mean(), result.rand_err.mean()
+        return pd.DataFrame(return_dict.copy())
     
     def dmat_filewriter(self,
                         pyfile,
@@ -870,11 +921,12 @@ class cognet:
                 self.set_nsamples(num_samples)
             
             tmp_path = "mpi_tmp/"
-            pd.DataFrame(self.samples_as_strings).to_csv(tmp_path+"tmp_samples_as_strings.csv", header=None, index=None)
-            w = self.samples.index.size
-            
             if not os.path.exists(tmp_path):
                 os.makedirs(tmp_path)
+            
+            pd.DataFrame(self.samples_as_strings).to_csv(tmp_path+"tmp_samples_as_strings.csv", header=None, index=None)
+            
+            w = self.samples.index.size
             with open(tmp_path+pyfile, 'w+') as f:
                 f.writelines(["from mpi4py.futures import MPIPoolExecutor\n",
                               "import numpy as np\n",
@@ -885,7 +937,7 @@ class cognet:
 
                 f.writelines(["w = {}\n".format(w),
                               "h = w\n",
-                              "p_all = pd.read_csv(\"tmp_samples_as_strings.csv\")\n\n"])
+                              "p_all = pd.read_csv(\"tmp_samples_as_strings.csv\", header=None)\n\n"])
 
                 f.writelines(["def distfunc(x,y):\n",
                               "\td=qdistance(x,y,qnet,qnet)\n",
@@ -903,24 +955,9 @@ class cognet:
                 f.writelines(["if __name__ == '__main__':\n",
                               "\twith MPIPoolExecutor() as executor:\n",
                               "\t\tresult = executor.map(dfunc_line, range(h))\n",
-                              "\t\tpd.DataFrame(result).to_csv(\'{}\',index=None,header=None)".format(OUTFILE)])
-
-            # with open(MPI_LAUNCHER_FILE, 'wx') as ml:
-            #     ml.writelines(["#!/bin/bash\n\n",
-            #                    "PROG=\"\"\n",
-            #                    "RSTR=`cat /dev/urandom | tr -dc \'a-zA-Z0-9\' | fold -w 20  | head -n 1 | cut -c 1-6`\n",
-            #                    "JOBN=IXC\"$RSTR\"\n",
-            #                    "TIME=10\n",
-            #                    "NCOR=28\n",
-            #                    "MEMR=10\n",
-            #                    "NODE=1\n",
-            #                    "EXECUTE=0\n",
-            #                    "DEPEND=""\n",
-            #                    "CDIR=`pwd`\n",
-            #                    "DRY_RUN=0\n",
-            #                    "ANYDEPEND=""\n",
-            #                    "PART='sandyb'\n"
-            #                    ])
+                              "\t\tresult = np.array(result)\n",
+                              "\t\tresult = pd.DataFrame(np.maximum(result, result.transpose()))\n"
+                              "\t\tresult.to_csv(\'{}\',index=None,header=None)".format(OUTFILE)])
                 
             with open(tmp_path+MPI_SETUP_FILE, 'w+') as ms:
                 ms.writelines(["#!/bin/bash\n",
@@ -964,7 +1001,6 @@ class cognet:
                                "\t./{} $yr $NODES $NUM tmp_\"$yr\"\n".format(MPI_SETUP_FILE),
                                "\t$LAUNCH -P tmp_\"$yr\" -F -T $T -N \"$NODES\" -C 28 -p broadwl -J ACRDALL_\"$yr\" -M 56\n",
                                "done\n",
-                               "rm tmp*\n"])
+                               "rm tmp_\"$yr\"*\n"])
         else:
             raise ValueError("load data first!")
-        print("running")
